 /*************************************************************************
 *                                                                        *
 *  tapeload - Emulate a Honeywell 5082 or similar paper tape reader      *
 *  Copyright (C) 2004 Philipp Hachtmann (hachti@hachti.de)               *
 *                                                                        *
 **************************************************************************
 *                                                                        *
 *  This program is free software; you can redistribute it and/or         *
 *  modify it under the terms of the GNU General Public License           *
 *  as published by the Free Software Foundation; either version 2        *
 *  of the License, or (at your option) any later version.                *
 *                                                                        *
 *  This program is distributed in the hope that it will be useful,       *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *  GNU General Public License for more details.                          *
 *                                                                        *
 *  You should have received a copy of the GNU General Public License     *
 *  along with this program; if not, write to the Free Software           *
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston,                *
 *  MA  02111-1307, USA.                                                  *
 *                                                                        *
 **************************************************************************
 *                                                                        *
 *  ABOUT                                                                 *
 *  This program emulates a parallel paper tape reader connected to	  *
 *  the honeywell H316 computer or similar.  I developed it to be more    *
 *  flexible while repairing my own H316 computer.                        *
 *                                                                        *
 *  All you need to emulate the tape reader is:                           *
 *  	- this program                                                    *
 *      - Linux pc 				                          *
 *      - 8255 compatible parallel port                                   *
 *      - d-sub cable connector 25 pin male                               *
 *      - a cable with many wires                                         *
 *      - "CablePAC PAC" card to connect to the controller               *
 *      - Nothing else :-)                                                *
 *                                                                        *
 **************************************************************************
 *                                                                        *
 *  CABLE                                                                 *
 *  I used a direct connection between the Tape reader control logic      *
 *  and my PC. And I know that this not ideal because of the different    *
 *  logic levels (6V H316 and 5V on the parallel port.                    *
 *  If you are afraid you can put some diodes between the port and the    *
 *  Honeywell.                                                            *
 *                                                                        *
 *                                                                        *
 *   PC	  	   direction	     Honeywell connector "C17"		  *
 *									  *    
 *   D0-D7   (2-9)	==>	     DATA (8-1)			          *
 *   ACK     (10) 	<==	     START (12)			          *	
 *   BUSY    (11)	<==	     STOP (14)                            *
 *   AUTOFED (14)       ==>	     ONLINE/READY (15)                    *
 *   STROBE  (1)	==>	     SPKR1 (13)                           *
 *   INIT    (16)	==>	     SPKR2 (11)                           *
 *                                                                        *
 **************************************************************************    
 *                                                                        *
 *   HOWTO                                                                *
 *   This program is dirty. You can't simply run it. It needs direct      *
 *   hardware access to the parallel port. You also should remove any     *
 *   drivers using the parallel port - if you have problems...            *
 *   If you try to run the program directly it will crash with segfault.  *
 *   To prevent this it mus be wrapped by my little perm program. I wrote *
 *   the perm program separately because it is easier to have one suid    *
 *   executable which stays unchanged while the user-owned rest can be    *
 *   made without ever becoming root again.                               *
 *                                                                        *
 **************************************************************************
 *                                                                        * 
 *   VERSION                                                              *
 *   2004-10-01 ver. 0.1 - The beginning.                                 *
 *                                                                        * 
 *************************************************************************/
 
// Enter your parallel port's base address
#define BASE_PORT 0x378    
 
#include <stdio.h>
#include <sys/io.h>
#include <sys/select.h>


void set_data(char data);
void set_online(char state);
void set_sprk(char state1, char state2);

int get_start();
int get_stop();

void set_data(char data){
    outb (data, BASE_PORT);
}


void set_online(char state){
    printf("set_online(%i)\n",state);
    char tmp=inb(BASE_PORT+2);
    if (!state) tmp=tmp|2;
    else tmp=tmp&(255-2);
    outb(tmp, BASE_PORT+2);
}
    

void set_sprk(char state1, char state2){
    char tmp;
    tmp=inb(BASE_PORT+2);
    
    
    if (state1) 
    tmp=tmp|1;
    else tmp=tmp&(255-1);
        
    if (state2) 
    tmp=tmp|4;
    else tmp=tmp&(255-4);
        
    outb(tmp, BASE_PORT+2);
}


int get_start(){
    if ((inb(BASE_PORT+1)&64)) return 1;
    else return 0;
}
    

int get_stop(){
    if (inb(BASE_PORT+1) & 128) return 0;
    else return 1;
}


void pause(int sec, int usec){
    struct timeval timeout;
    timeout.tv_sec=sec;
    timeout.tv_usec=usec;
    select(0,0,0,0,&timeout);
}


unsigned char twist(unsigned char in){
    int c;
    char out=0;
    for (c=0; c<8; c++){
	out= out << 1;
	if (in & 1) out+=1;
	in = in >> 1;
    }	
    return out;
}


void debug_thing(){
    while(1) {
	printf("+1:%i  +2:%i \n",inb(BASE_PORT+1), inb(BASE_PORT+2));
	printf("get_start:%i get_stop:%i \n",get_start(),get_stop());
	pause(0,300000);
    }
}    


int main( int argc, char * argv[]){
    
    set_data(0);
    set_online(1);
    set_sprk(0,0);
    
    if (argc==2) debug_thing();
    printf("get_start:%i get_stop:%i \n",get_start(),get_stop());
    printf("\nParport-loader bereit. Enter fr \"Einschalten\".\n");
    //getc(stdin);
    
    set_online(1);
    printf("Leser ist online geschaltet.\n");
    printf("get_start:%i get_stop:%i \n",get_start(),get_stop());
    pause(1,0);
    printf("get_start:%i get_stop:%i \n",get_start(),get_stop());
while(1){
    
        while (get_start()==0)pause(0,1);
        printf("Start erkannt.\n");
    
        //pause(0,100000); // Leser luft an.
    
    

        while (get_start()){	
    	    char buf;
	    if (read(0,&buf,1)==0){
		printf("Ende der Eingabe. Tsch.\n");
		exit(0);
	    }
	    buf=twist(buf);
	    
	    printf("Output: %u\n",buf);
	    set_data(buf);
	    pause(0,500); // 1/100000 Sekunde, dann kommen die Strobes:
	
	    set_sprk(0,1);	
	    pause(0,1000); // 1/100 Sekunde fr das Strobing.
	    set_sprk(1,0);

	    pause(0,1000); // 1/1000 Sek. bleiben die Daten noch gltig.
	    set_data(0);  // Jetzt sind die Daten weg.
	
	    pause (0,10000); // 1/1000 Sek. Pause bis zum nchsten Zeichen.
	    //printf("get_start:%i get_stop:%i \n",get_start(),get_stop());
	}
	printf("Stop erkannt.\n");

    } 

    return 0;
} //main()


 /************************************************************************/
 /* TheEnd. */
